//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "types.h"
#include "partfile.h"
#include "memdc.h"
#include "titlemenu.h"

#include <map>

using namespace std;

class CMuleCtrlItem
{
public:
	// Constructor
	CMuleCtrlItem(CPartFile* pFile, CUpDownClient* pSource)
		{ m_pFile = pFile; m_pSource = pSource; }

	// Methods
	bool IsFile(){ return m_pSource == NULL; }
	bool IsSource(){ return m_pSource != NULL; }
	CPartFile* GetFile(){ return m_pFile; }
	CUpDownClient* GetSource(){ return m_pSource; }
	bool IsAskedForAnotherFile();

public:
	// Public data (too lazy to hide :)
	DWORD	m_dwUpdated;
	CBitmap	m_Status;

private:
	CPartFile*		m_pFile;
	CUpDownClient*	m_pSource;
};

typedef multimap<void*, void*> MuleMultiMap;
typedef MuleMultiMap::iterator MuleIter;
typedef pair<MuleIter, MuleIter> MuleRange;

class CMuleItemMgr
{
public:
	virtual ~CMuleItemMgr();

	// File
	CMuleCtrlItem* AddFile(CPartFile* pFile);
	void RemoveFile(CMuleCtrlItem* pFile);
	CMuleCtrlItem* FindFile(CPartFile* pFile);
	MuleRange GetAllFiles();
	MuleRange GetItemsByFile(CPartFile* pFile);

	// Source
	CMuleCtrlItem* AddSource(CUpDownClient* pSource, CPartFile* pFile);
	void RemoveSource(CMuleCtrlItem* pSource);
	void RemoveSource(CUpDownClient* pSource);
	CMuleCtrlItem* FindSource(CUpDownClient* pSource, CPartFile* pFile, BOOL bLock = TRUE);

	// Lock
	void LockRemove(){ m_MutexRemove.Lock(); }
	void UnlockRemove(){ m_MutexRemove.Unlock(); }

private:
	MuleMultiMap m_ListData;	// NxN list, files and sources vs. data

	CMutex m_MutexAccess;
	CMutex m_MutexRemove;
};


// CDownloadListCtrl
class CDownloadListCtrl : public CMuleListCtrl
{
	DECLARE_DYNAMIC(CDownloadListCtrl)

public:
	CDownloadListCtrl();
	virtual ~CDownloadListCtrl();
	virtual void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct);
	void	UpdateItem(void* toupdate);
	void	Init();
	void	AddFile(CPartFile* toadd);
	void	AddSource(CPartFile* owner,CUpDownClient* source,bool notavailable);
	void	RemoveSource(CUpDownClient* source,CPartFile* owner);
	void	RemoveFile(CPartFile* toremove);
	void	ClearCompleted();
	virtual BOOL OnCommand(WPARAM wParam,LPARAM lParam );
	void	SetStyle();
	void	CreateMenues();
	void	Localize();
	void	RestartWaitingDownloads(); 				// Rod
	void	RefreshList();						// Cax2
	void	SortInit(int override);					// Cax2
	bool	IsShowingSources() {return m_ShowSrc;}			// Cax2
	int8	FilesSortIndex;
	void	HideSources(CPartFile* partfile);  			// Cax2 closefile bugfix
	void	DownloadAllA4AF(CPartFile* file); 			// Tarod
	CString getTextList();


protected:
	void	DrawFileItem(CDC *dc, int nColumn, LPRECT lpRect, CMuleCtrlItem *lpCtrlItem);
	void	DrawSourceItem(CDC *dc, int nColumn, LPRECT lpRect, CMuleCtrlItem *lpCtrlItem);
	afx_msg void OnLvnItemActivate(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnNMRclick(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg	void OnColumnClick( NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnListModified(NMHDR *pNMHDR, LRESULT *pResult);
	static int CALLBACK SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
    static int Compare(CPartFile* file1, CPartFile* file2, LPARAM lParamSort);
	static int Compare(CUpDownClient* client1,CUpDownClient* client2, LPARAM lParamSort, int sortMod);

	DECLARE_MESSAGE_MAP()
private:
	CImageList  	imagelist;
	CTitleMenu		m_ClientMenu;
	CMenu			m_PrioMenu;
	CMenu			m_MovieMenu;		// DonGato
	CTitleMenu		m_FileMenu;
	int         	m_CurrentSortItem;
	bool        	m_SortAscending;
	bool			m_ShowSrc;  		// Cax2 
	int				m_lastSort;
	DWORD			m_sortTime;

	CMuleItemMgr	m_ItemMgr;

public:
	afx_msg void OnNMClick(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnNMDblclkDownloadlist(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	void RefreshInfo(void);
};
